<?php

declare(strict_types=1);

namespace Drupal\sobki_admin\Form;

use Drupal\Component\Transliteration\TransliterationInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Template\Attribute;
use Drupal\Core\Url;
use Drupal\sobki_admin\Service\LayoutContextHelperInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a class for reacting to form operations.
 */
class FormOperations implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected RendererInterface $renderer;

  /**
   * The entity type bundle info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected EntityTypeBundleInfoInterface $entityTypeBundleInfo;

  /**
   * The layout context helper.
   *
   * @var \Drupal\sobki_admin\Service\LayoutContextHelperInterface
   */
  protected LayoutContextHelperInterface $layoutContextHelper;

  /**
   * The transliteration service.
   *
   * @var \Drupal\Component\Transliteration\TransliterationInterface
   */
  protected TransliterationInterface $transliteration;

  /**
   * A list of UI Styles wrappers to be processed before render.
   *
   * @var string[]
   */
  private array $uiStylesElements;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entityTypeBundleInfo
   *   The entity type bundle info service.
   * @param \Drupal\sobki_admin\Service\LayoutContextHelperInterface $layoutContextHelper
   *   The layout context helper.
   * @param \Drupal\Component\Transliteration\TransliterationInterface $transliteration
   *   The transliteration service.
   */
  public function __construct(
    RendererInterface $renderer,
    EntityTypeBundleInfoInterface $entityTypeBundleInfo,
    LayoutContextHelperInterface $layoutContextHelper,
    TransliterationInterface $transliteration
  ) {
    $this->renderer = $renderer;
    $this->entityTypeBundleInfo = $entityTypeBundleInfo;
    $this->layoutContextHelper = $layoutContextHelper;
    $this->transliteration = $transliteration;

    $this->uiStylesElements = [
      'ui_styles_wrapper',
      'ui_styles_title',
      'ui_styles',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    // @phpstan-ignore-next-line
    return new static(
      $container->get('renderer'),
      $container->get('entity_type.bundle.info'),
      $container->get('sobki_admin.layout_context_helper'),
      $container->get('transliteration'),
    );
  }

  /**
   * Alters forms for our Sobki customizations.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param string $form_id
   *   The form ID.
   *
   * @see hook_form_alter()
   */
  public function formAlter(array &$form, FormStateInterface $form_state, string $form_id): void {
    // layout_builder_configure_section.
    // layout_builder_add_block.
    if ($form_id === 'layout_builder_add_block' || $form_id === 'layout_builder_update_block') {
      // Primary tabs.
      $form['sobki_primary_tabs'] = [
        '#theme' => 'item_list',
        '#list_type' => 'ul',
        '#items' => [
          $this->getListElement('Content', 'sobki-primary-tab-content', 'sobki-primary-tab', 'active'),
          $this->getListElement('Styles', 'sobki-primary-tab-styles', 'sobki-primary-tab'),
        ],
        '#attributes' => [
          'class' => [
            'sobki-tab',
            'sobki-primary-tab',
            'nav',
            'nav-pills',
            'glb-claro-details__wrapper',
          ],
        ],
        '#weight' => (int) -99,
      ];

      // Secondary tabs.
      $secondary_tabs = [
        '#theme' => 'item_list',
        '#list_type' => 'ul',
        '#items' => [
          $this->getListElement('Block', 'sobki-secondary-tab-block', 'sobki-secondary-tab', 'active'),
          $this->getListElement('Title', 'sobki-secondary-tab-title', 'sobki-secondary-tab'),
          $this->getListElement('Inner content', 'sobki-secondary-tab-content', 'sobki-secondary-tab'),
        ],
        '#attributes' => [
          'class' => [
            'sobki-tab',
            'sobki-secondary-tab',
            'nav',
            'glb-claro-details__wrapper',
          ],
          'data-drupal-selector' => [
            'edit-sobki-primary-tabs',
          ],
        ],
      ];

      // Wrappers & script attachment.
      $form['settings']['#prefix'] = '<div id="sobki-primary-tab-content" class="sobki-primary-tab-wrapper">';
      $form['settings']['#suffix'] = '</div>';
      $form['ui_styles_wrapper']['#prefix'] = '<div id="sobki-primary-tab-styles" class="sobki-primary-tab-wrapper">' . $this->renderer->render($secondary_tabs) . '<div id="sobki-secondary-tab-block" class="sobki-secondary-tab-wrapper">';
      $form['ui_styles_wrapper']['#suffix'] = '</div>';
      $form['ui_styles_title']['#prefix'] = '<div id="sobki-secondary-tab-title" class="sobki-secondary-tab-wrapper">';
      $form['ui_styles_title']['#suffix'] = '</div>';
      $form['ui_styles']['#prefix'] = '<div id="sobki-secondary-tab-content" class="sobki-secondary-tab-wrapper">';
      $form['ui_styles']['#suffix'] = '</div></div>';
      $form['#attached']['library'][] = 'sobki_admin/sobki_admin_tabs';
      $form['#attached']['library'][] = 'sobki_admin/sobki_admin_selecticons_opacity';
      $form['#attached']['library'][] = 'sobki_admin/sobki_select_items';
      $form['#attached']['library'][] = 'sobki_admin/sobki_tabs';

      // Add a separator between "block title" section and "block content"
      // section.
      if (isset($form['settings']['label_display'])) {
        $form['settings']['label_display']['#suffix'] = '<hr>';
      }

      foreach ($this->uiStylesElements as $element) {
        // Force open all UI Styles fieldsets.
        $form[$element]['#title'] = '';
        $form[$element]['#type'] = 'fieldset';
        $form[$element]['#collapsible'] = FALSE;
        $form[$element]['#collapsed'] = FALSE;

        // Reorder elements.
        $order = (int) -99;
        // cspell:disable
        $children_order = [
          'text', 'link', 'background', 'borders', 'spacing',
          'position', 'sizing', 'visibility', 'display',
          'reel_float_', 'flex', 'object_fit', 'z_index',
          'interactions', 'focus', 'vignette', 'other', '_ui_styles_extra',
        ];
        $children_order = \array_map([$this, 'getMachineName'], $children_order);
        // cspell:enable
        foreach ($children_order as $child) {
          $form[$element][$child]['#weight'] = $order;
          ++$order;
        }

        // Open first elements.
        $opened_children = [];
        // @phpstan-ignore-next-line
        foreach ($opened_children as $child) {
          $form[$element][$child]['#open'] = TRUE;
        }

        // Use SelectIcons on styles select.
        $selecticons_children = [
          $this->getMachineName('text') => [
            'ui_styles_colors_color',
            'ui_styles_colors_opacity',
          ],
          $this->getMachineName('link') => [
            'ui_styles_link_colored',
            'ui_styles_link_underline',
            'ui_styles_link_opacity',
            'ui_styles_link_underline_opacity',
            'ui_styles_link_opacity_hover',
            'ui_styles_link_underline_opacity_hover',
          ],
          $this->getMachineName('background') => [
            'ui_styles_colors_background_color',
            'ui_styles_colors_background_opacity',
          ],
          $this->getMachineName('borders') => [
            'ui_styles_borders_border_color',
            'ui_styles_borders_border_opacity',
          ],
        ];
        foreach ($selecticons_children as $child => $subchildren) {
          foreach ($subchildren as $subchild) {
            $form[$element][$child][$subchild]['#type'] = 'select_icons';

            // Create options attributes.
            $options_attributes = [];
            foreach ($form[$element][$child][$subchild]['#options'] as $key => $value) {
              $background = \explode('-', $key);
              \array_shift($background);
              $background = 'bg-' . \implode('-', $background);
              $options_attributes[$key] = new Attribute([
                'data-class' => [
                  $key,
                  $background,
                  'same-background-color',
                ],
              ]);
            }
            $form[$element][$child][$subchild]['#options_attributes'] = $options_attributes;
          }
        }

        // Use checkboxes on simple fields.
        foreach (Element::children($form[$element]) as $child) {
          if ($form[$element][$child]['#type'] == 'details') {
            $form[$element][$child]['#attributes']['class'][] = 'glb-claro-details__wrapper';
            foreach (Element::children($form[$element][$child]) as $subchild) {
              if ($form[$element][$child][$subchild]['#type'] == 'select' && \count($form[$element][$child][$subchild]['#options']) === 1) {
                $form[$element][$child][$subchild]['#type'] = 'checkbox';
                // We are forcing the validation status as we can not validate
                // the checkbox value before the core validation.
                // This way we ensure the checkbox won't block the submission.
                $form[$element][$child][$subchild]['#validated'] = TRUE;
              }
            }
          }
        }

        // Change select to radio button.
        // cspell:disable.
        $radios_children = [
          $this->getMachineName('text') => [
            'ui_styles_text_font_italics',
            'ui_styles_text_text_alignment',
            'ui_styles_text_text_transform',
            'ui_styles_text_text_wrapping',
            'ui_styles_text_text_decoration',
          ],
          $this->getMachineName('borders') => [
            'ui_styles_borders_border',
          ],
          $this->getMachineName('position') => [
            'ui_styles_position_center',
          ],
          $this->getMachineName('visibility') => [
            'ui_styles_visibility',
          ],
          $this->getMachineName('flex') => [
            'ui_styles_flex_align_content',
            'ui_styles_flex_align_items',
            'ui_styles_flex_align_self',
            'ui_styles_flex_justify_content',
            'ui_styles_flex_wrap',
          ],
        ];
        // phpcs:ignore
        // cspell:enable.

        foreach ($radios_children as $child => $subchildren) {
          foreach ($subchildren as $subchild) {
            $form[$element][$child][$subchild]['#type'] = 'radios';
            foreach ($form[$element][$child][$subchild]['#options'] as $key => $option) {
              $form[$element][$child][$subchild]['#options'][$key] = '<i class="' . $key . '-icon"></i><span>' . $option . '</span>';
            }
          }
        }
      }
    }

    if ($form_id === 'node_page_layout_builder_form' || $form_id === 'entity_view_display_layout_builder_form') {
      /** @var \Drupal\layout_builder\Form\UpdateBlockForm $form_object */
      $form_object = $form_state->getFormObject();
      $contexts = $form_object->getSectionStorage();
      $plugin_id = $contexts->getPluginId();

      /** @var string $entity_type_id */
      $entity_type_id = $this->layoutContextHelper->getHostEntityTypeId($contexts);
      $entity_bundle_id = $this->layoutContextHelper->getHostEntityBundle($contexts);

      $bundle_label = $this->entityTypeBundleInfo
        ->getBundleInfo($entity_type_id)[$entity_bundle_id]['label'];

      $tab_items = [];
      $tab_items[] = [
        '#markup' => '<div class="nav-link"><b>' . $bundle_label . '</b></div>',
        '#wrapper_attributes' => [
          'class' => [
            'nav-item',
          ],
        ],
      ];
      if ($entity_type_id == 'node') {
        $layout_url = Url::fromRoute('layout_builder.defaults.' . $entity_type_id . '.view')
          ->setRouteParameters([
            'node_type' => $entity_bundle_id,
            'view_mode_name' => 'full',
          ]);
        $layout_link = Link::fromTextAndUrl($this->t('Manage template'), $layout_url)->toString();

        if ($plugin_id === 'overrides') {
          $tab_items[] = [
            '#markup' => '<div class="nav-link active">' . $this->t('Content') . '</div>',
            '#wrapper_attributes' => [
              'class' => [
                'nav-item',
                'active',
              ],
            ],
          ];
          $tab_items[] = [
            '#markup' => '<div class="nav-link">' . $layout_link . '</div>',
            '#wrapper_attributes' => [
              'class' => [
                'nav-item',
              ],
            ],
          ];
        }
        else {
          $tab_items[] = [
            '#markup' => '<div class="nav-link active">' . $this->t('Template') . '</div>',
            '#wrapper_attributes' => [
              'class' => [
                'nav-item',
                'active',
              ],
            ],
          ];
        }
      }

      // Primary tabs.
      $form['sobki_primary_tabs'] = [
        '#theme' => 'item_list',
        '#list_type' => 'ul',
        '#items' => $tab_items,
        '#attributes' => [
          'class' => [
            'nav',
            'nav-pills',
            'glb-claro-details__wrapper',
          ],
        ],
        '#weight' => (int) -9999,
      ];

      $form['revision_information']['#open'] = FALSE;
      unset($form['layout_builder_message']);
    }
  }

  /**
   * Get the style category machine name in the proper language.
   *
   * @see: Drupal\ui_styles\MachineNameTrait
   */
  protected function getMachineName(string $string): string {
    // phpcs:ignore Drupal.Semantics.FunctionT.NotLiteralString
    $string = $this->t(\ucfirst($string));
    $transliterated = $this->transliteration->transliterate($string, LanguageInterface::LANGCODE_DEFAULT, '_');
    $transliterated = \mb_strtolower($transliterated);
    $transliterated = \preg_replace('@[^a-z0-9_.]+@', '_', $transliterated);
    return $transliterated ?? '';
  }

  /**
   * Formats a proper item list array for usage in the render array.
   *
   * @param string $title
   *   The link title.
   * @param string $fragment
   *   The anchor on which the link points.
   * @param string $class
   *   A CSS class.
   * @param string $active
   *   Active class to apply to the element.
   *
   * @return array
   *   The list element.
   */
  private function getListElement(string $title, string $fragment, string $class, string $active = ''): array {
    $url = Url::fromRoute('<none>')->setOptions([
      'fragment' => $fragment,
      'attributes' => [
        'id' => 'link-' . $fragment,
        'class' => [
          'nav-link',
          $class,
          $active,
        ],
      ],
    ]);
    // phpcs:ignore Drupal.Semantics.FunctionT.NotLiteralString
    $link = Link::fromTextAndUrl($this->t($title), $url)->toString();
    return [
      '#markup' => $link,
      '#wrapper_attributes' => [
        'class' => [
          'nav-item',
        ],
      ],
    ];
  }

}
