<?php

declare(strict_types=1);

namespace Drupal\sobki_tablefield\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\ui_patterns\Form\PatternDisplayFormTrait;
use Drupal\ui_patterns_field_formatters\Plugin\Field\FieldFormatter\PatternOneForAllFormatter;

/**
 * Plugin implementation of the Sobki Tablefield formatter.
 *
 * @FieldFormatter(
 *     id = "sobki_tablefield",
 *     label = @Translation("SOBKI Tabular View"),
 *     field_types = {
 *         "tablefield"
 *     }
 * )
 */
class SobkiTablefieldFormatter extends PatternOneForAllFormatter implements ContainerFactoryPluginInterface {

  use PatternDisplayFormTrait;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $settings = parent::defaultSettings();
    $settings['row_header'] = 1;
    return $settings;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    // Prevent nesting (unsupported yet).
    unset($form['type']['#options']['pattern_all_formatter'], $form['type']['#options']['pattern_each_formatter']);

    $form['row_header'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display first row as a table header'),
      '#default_value' => $this->getSetting('row_header'),
    ];

    $field_storage_definition = $this->fieldDefinition->getFieldStorageDefinition();
    $context = [
      'storageDefinition' => $field_storage_definition,
      'limit' => $field_storage_definition->getPropertyNames(),
    ];

    // Some modifications to make 'variant' default value working.
    $configuration = $this->getSettings();

    $this->buildPatternDisplayForm($form, 'field_properties', $context, $configuration);

    $form['pattern']['#attributes']['disabled'] = 'disabled';
    $form['variants']['table']['#attributes']['disabled'] = 'disabled';
    $form['#element_validate'] = [[static::class, 'cleanSettings']];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    $row_header = $this->getSetting('row_header');
    if ($row_header) {
      $summary[] = (string) $this->t('First row as a table header');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   *
   * @SuppressWarnings(PHPMD.CyclomaticComplexity)
   * @SuppressWarnings(PHPMD.NPathComplexity)
   */
  public function viewElements(FieldItemListInterface $items, $langcode = NULL) {
    $entity = $items->getEntity();
    $elements = [];
    $header = [];
    $tabledata = [];
    $caption = '';
    $row_header = $this->getSetting('row_header');
    $pattern = $this->getSetting('pattern');

    foreach ($items as $table) {
      if (!empty($table->value)) {
        // Tablefield::rationalizeTable($table->value);.
        $tabledata = $table->value;
        $caption = $tabledata['caption'];
        unset($tabledata['caption']);

        // Run the table through input filters.
        foreach ($tabledata as $row_key => $row) {
          foreach ($row as $col_key => $cell) {
            if (\is_numeric($col_key)) {
              $tabledata[$row_key]['cells'][$col_key] = [
                'content' => empty($table->format) ? $cell : \check_markup($cell, $table->format),
              ];
            }
            else {
              // Do not show special extra columns like weight.
              unset($tabledata[$row_key]['cells'][$col_key]);
            }
          }
        }
      }
    }

    if ($row_header) {
      // Pull the header for theming.
      $header_data = \array_shift($tabledata);

      // Check for an empty header, if so we don't want to theme it.
      $has_header = FALSE;
      if (\is_array($header_data) && isset($header_data['cells'])) {
        foreach ($header_data['cells'] as $cell) {
          if ($cell['content'] !== '') {
            $has_header = TRUE;
            break;
          }
        }
      }
      if ($has_header) {
        $header = $header_data['cells'];
      }
    }

    // Set pattern render array.
    $elements[0] = [
      '#type' => 'pattern',
      '#id' => $this->getSetting('pattern'),
      '#context' => [
        'entity' => $entity,
        'type' => 'field_formatter',
        'formatter' => [
          'id' => $this->getPluginId(),
          'class' => static::class,
        ],
        'items' => $items,
        'item' => $items[0] ?? NULL,
      ],
      '#fields' => [
        'rows' => $tabledata,
        'header' => $header,
        'caption' => $caption,
      ],
    ];

    // Set the variant.
    /** @var string $pattern_variant */
    $pattern_variant = $this->getSetting('pattern_variant');
    if (!empty($pattern_variant)) {
      $elements[0]['#variant'] = $pattern_variant;
    }

    // Set the settings.
    /** @var array $settings */
    $settings = $this->getSetting('pattern_settings');
    $pattern_settings = !empty($settings) && isset($settings[$pattern]) ? $settings[$pattern] : NULL;
    if (!empty($pattern_settings)) {
      $elements[0]['#settings'] = $pattern_settings;
    }

    // Set the variant tokens.
    /** @var array $variant_tokens */
    $variant_tokens = $this->getSetting('variants_token');
    $variant_token = !empty($variant_tokens) && isset($variant_tokens[$pattern]) ? $variant_tokens[$pattern] : NULL;
    if (!empty($variant_token)) {
      $elements[0]['#variant_token'] = $variant_token;
    }

    return $elements;
  }

}
